<?php
#
# Script         phpErrorReporter
# Version        1.0.1
# Date           Apr 19, 2013
# Copyright      2011 - 2013 Denny Schlesinger
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# Autor          Denny Schlesinger
#
# $Id: $
#
/********************************************************************
phpErrorReporter will log php errors and warnings and it will send email 
alerts to the designated email address in case of fatal php errors.

phpErrorReporter is activated as an auto-prepend-file in htaccess
See instructions below.

phpErrorReporter uses two functions because the user error handler does not
(can not) catch fatal errors

shutdownReporter() <-- handles fatal errors
errorReporter()    <-- handles all other errors

If you use custom error pages, check out the error500.php example.

Enjoy!

Denny Schlesinger
*********************************************************************

# htaccess for phpErrorReporter

# activate phpErrorReporter
php_value auto_prepend_file /full/path/to/phpErrorReporter.php

# php error handling for production servers
php_flag display_errors off
php_flag log_errors on
php_value error_log /full/path/to/phpError.log

# other php settings
# php_flag display_startup_errors off
# php_flag html_errors off
# php_flag ignore_repeated_errors off
# php_flag ignore_repeated_source off
# php_flag report_memleaks on
# php_flag track_errors on
# php_value docref_root 0
# php_value docref_ext 0
# [see footnote 3] # php_value error_reporting 999999999
# php_value error_reporting -1
# php_value log_errors_max_len 0
********************************************************************/

// Parameters used by the phpErrorReporter functions
// Constants are safer than global variables

// Make sure the error log path is the same as used in .htaccess
define("PHP_ERROR_REPORTER_LOG_FILE_PATH", "/full/path/to/phpError.log");

// Email address of webmaster or administrator
define("PHP_ERROR_REPORTER_EMAIL_TO", "webmaster@{$_SERVER['SERVER_NAME']}");

// Extra email headers: email from
define("PHP_ERROR_REPORTER_EMAIL_FROM", "From: phpErrorReporter@{$_SERVER['SERVER_NAME']}");

// Email intro
define("PHP_ERROR_REPORTER_EMAIL_INTRO", "Please do not reply to this email.\nThis is an automated message from phpErrorReporter at the indicated domain\n\n");

// Browser feed back top line
define("PHP_ERROR_REPORTER_FEEDBACK1", "<!DOCTYPE html>
<html>
<head>
<meta http-equiv='content-type' content='text/html; charset=iso-8859-1'>
<title>Error Report | {$_SERVER['SERVER_NAME']}</title>
<style type='text/css'>
h2 {
  font-family:Verdana, Helvetica, Arial, sans-serif;
  font-size:1.5em;
}
</style>
</head>
<body>
<h2>This system is temporarily unavailable</h2>
<p>The following has been reported to the administrator:</p>\n");

// Browser feed back bottom line
define("PHP_ERROR_REPORTER_FEEDBACK2", "<p>Sorry for the inconvenience.</p>
<div style=\"margin-top:50px;font-size:0.9em;\">
  <strong>phpErrorReporter.php</strong> v 1.0.1<br>
  <a href=\"http://softwaretimes.com/downloads/update+phperrorreporter+v+.html\" target=\"_blank\">The Ultimate Error Reporter</a><br>
  Copyright 2011 - 2013, Denny Schlesinger<br>
</div>
</body>
</html>\n");

////////////////////////////////////////////////////////////////


// set timezone

// WARNING: If you don't set a valid time zone the
//          phpErrorReporter script will fail
// List of Supported Timezones
// http://www.php.net/manual/en/timezones.php

// use default server timezone
// $timeZone = date_default_timezone_get();
// or set custom timezone
$timeZone = 'America/Caracas';

if (!date_default_timezone_set($timeZone)) {
    unset ($timeZone);  // clean up!
    trigger_error ('Invalid timezone', E_USER_ERROR);
}

// clean up!
unset ($timeZone);

////////////////////////////////////////////////////////////////


// register error handlers

// For fatal errors that cannot be handled by the user error handler
// The error is logged by php so no need to do it here
// Just send the email alert and the browser feedback
register_shutdown_function('shutdownReporter');

// For errors that can be handled with a user function
// This disables php's error handler so we have to do it all
// For warnings, log the error
// For fatal errors, log the error, 
// send the email alert and the browser feedback
set_error_handler('errorReporter');

////////////////////////////////////////////////////////////////


function shutdownReporter() {

    // unset errorReporter()
    // comment out while testing this function
    restore_error_handler();
    
    // error types to report
    $haystack = array(1, 4, 16, 32, 64, 128);

    $time = date('d-M-Y H:i:s');

    // get last error
    $error = error_get_last();

    // if fatal error occurred
    if(in_array ($error['type'], $haystack)) {

        // send email to administrator
        $email  = PHP_ERROR_REPORTER_EMAIL_INTRO;
        $email .= "Domain: {$_SERVER['SERVER_NAME']}\n";
        $email .= "PHP Fatal error\n";
        $email .= "Time: {$time}\n";        
        $email .= "Error: {$error['message']}\n";
        $email .= "File: {$error['file']} on line {$error['line']}";
        $email .= "\n\nError type {$error['type']}";
        error_log($email, 1, PHP_ERROR_REPORTER_EMAIL_TO, PHP_ERROR_REPORTER_EMAIL_FROM);
    
        // browser feedback
        $pos = strpos($error['message'], "'/home/");
        if($pos !== FALSE) {
            $error['message'] = substr($error['message'], 0, $pos) . "file";
        }
        $feedback  = PHP_ERROR_REPORTER_FEEDBACK1;
        $feedback .= "<strong style='color:red;'>{$error['message']}</strong>\n";
        $feedback .= PHP_ERROR_REPORTER_FEEDBACK2;
        echo $feedback;
        
    } // if(in_array ($error['type'], $haystack))
    
} // function shutdownReporter()


function errorReporter($errno, $errstr, $errfile, $errline, $errcontext) {

   $time = date('d-M-Y H:i:s');
   
   # for backwad compatibility with php 5.3.0
   if(!defined('E_COMPILE_ERROR'))     { define('E_COMPILE_ERROR', 64); }
   if(!defined('E_COMPILE_WARNING'))   { define('E_COMPILE_WARNING', 128); }
   if(!defined('E_RECOVERABLE_ERROR')) { define('E_RECOVERABLE_ERROR', 4096); }
   if(!defined('E_DEPRECATED'))        { define('E_DEPRECATED', 8192); }
   if(!defined('E_USER_DEPRECATED'))   { define('E_USER_DEPRECATED', 16384); }
   
   switch ($errno) {
      //                           "no" means we cannot handle this error
      //   error                   error number
      case E_ERROR:             // 1   - no
      case E_WARNING:           // 2
      case E_PARSE:             // 4   - no
      case E_NOTICE:            // 8
      case E_CORE_ERROR:        // 16  - no
      case E_CORE_WARNING:      // 32  - no
      case E_COMPILE_ERROR:     // 64  - no
      case E_COMPILE_WARNING:   // 128 - no
      case E_USER_WARNING:      // 512
      case E_USER_NOTICE:       // 1024
//      case E_STRICT:            // 2048 (it clutters up the error log)
      case E_DEPRECATED:        // 8192
      case E_USER_DEPRECATED:   // 16384
         // the above errors will be logged using php system format
         $logEntry = "PHP Warning: {$errstr} in {$errfile} on line {$errline}\n";
         error_log("[{$time}] {$logEntry}", 3, PHP_ERROR_REPORTER_LOG_FILE_PATH);
         break;

      // the following errors will:
      // 1) be logged using the php system format
      // 2) trigger an email alert
      // 3) output user feedback  to browser
      // 4) kill the php script
      case E_USER_ERROR:        // 256
      case E_RECOVERABLE_ERROR: // 4096
      
         // log error
         if($errstr === 'Error 500') {
            // log error 500
            $logEntry = "Error 500 {$_SERVER['REMOTE_ADDR']} http://{$_SERVER['HTTP_HOST']}{$_SERVER['REQUEST_URI']}\n";
         } else {
            // log the error using php system format
            $logEntry = "PHP Fatal error: {$errstr} in {$errfile} on line {$errline}\n";
         }
         error_log("[{$time}] {$logEntry}", 3, PHP_ERROR_REPORTER_LOG_FILE_PATH);
         
         // send email to administrator
         if($errstr === 'Error 500') {
            $email  = PHP_ERROR_REPORTER_EMAIL_INTRO;
            $email .= "Domain: {$_SERVER['SERVER_NAME']}\n";
            $email .= "Time: {$time}\n";        
            $email .= "Error 500 - Internal Server Error\n";
            $email .= "Visitor: {$_SERVER['REMOTE_ADDR']}\n"; 
            $email .= "http://{$_SERVER['HTTP_HOST']}{$_SERVER['REQUEST_URI']}";
         } else {
            $email  = PHP_ERROR_REPORTER_EMAIL_INTRO;
            $email .= "Domain: {$_SERVER['SERVER_NAME']}\n";
            $email .= "PHP Fatal error\n";
            $email .= "Time: {$time}\n";        
            $email .= "Error: {$errstr}\n";
            $email .= "File: {$errfile} on line {$errline}";
         }
         error_log($email, 1, PHP_ERROR_REPORTER_EMAIL_TO, PHP_ERROR_REPORTER_EMAIL_FROM);

         // terminate session, if any
         session_start();
         session_unset();
         session_destroy();

         // browser feedback
         if($errstr === 'Error 500') {
            $errstr .= " - Internal Server Error";
         }
         $feedback  = PHP_ERROR_REPORTER_FEEDBACK1;
         $feedback .= "<strong style='color:red;'>{$errstr}</strong>\n";
         $feedback .= PHP_ERROR_REPORTER_FEEDBACK2;
         echo $feedback;
         
         die();
         break;
      
   } // switch ($errno)
} // function errorReporter()

?>
